
package com.sample;

import java.nio.ByteBuffer;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.RemoteException;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Toast;

import com.felicanetworks.falp.FalpListener;
import com.felicanetworks.falp.FalpServiceConst;

public class FelicaFALPInitiatorActivity extends Activity implements OnClickListener {

    private AlertDialog mDialog;

    private FelicaFALPInitiatorSender sender;

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.falp_initiator);

        Button btn1 = (Button) findViewById(R.id.btn1);

        btn1.setOnClickListener(this);
    }

    private InitiatorTask task;

    @Override
    public void onClick(View v) {

        // 送信テキスト
        String text = "じゅげむ じゅげむ ごこうのすりきれ かいじゃりすいぎょの すいぎょうまつ うんらいまつ ふうらいまつ くうねるところにすむところ やぶらこうじのぶらこうじ ぱいぽ ぱいぽ ぱいぽのしゅーりんがん しゅーりんがんのぐーりんだい ぐーりんだいのぽんぽこぴーの ぽんぽこなーの ちょうきゅうめいのちょうす";

        sender = new FelicaFALPInitiatorSender(this);

        // ダイアログ表示
        mDialog = new
                AlertDialog.Builder(FelicaFALPInitiatorActivity.this).setMessage("端末をタッチさせてください")
                        .setNegativeButton("キャンセル", new
                                DialogInterface.OnClickListener() {

                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        // キャンセルボタンが押されたら、キャンセル処理を行う
                                        sender.canceled();

                                    }
                                }).create();

        mDialog.show();

        task = new InitiatorTask();
        task.execute(text);

    }

    @Override
    protected void onStop() {
        super.onStop();

        if (task != null) {
            task.cancel(true);
        }
    }

    class InitiatorTask extends AsyncTask<String, Integer, Void> {

        private boolean condition;
        private byte[] sendData;
        private AlertDialog dialog;
        // 次に送信する際の先頭のIndex
        private int sendPos;
        
        private static final int SUCESS_PROPOSEDINITIATORFALP = 0;
        private static final int SUCESS_COMPLETEDINITIATORFALP = 1;

        FalpListener falpListener = new FalpListener.Stub() {

            @Override
            public void completedInitiatorFalp() throws RemoteException {
                // FALPのハンドシェイク処理が完了

                // 通信中ダイアログ表示
                publishProgress(SUCESS_COMPLETEDINITIATORFALP);

            }

            @Override
            public void proposedInitiatorFalp() throws RemoteException {
                // FeliCaチップ捕捉完了
                sendData(sender.getMaxSendSize());

                publishProgress(SUCESS_PROPOSEDINITIATORFALP);
            }

            @Override
            public void receivedFalpHandshake() throws RemoteException {

            }

            @Override
            public void recvDataNotify(byte[] recvData, int len) throws RemoteException {

                // Targetから通知された受信完了サイズを取得する
                ByteBuffer buf = ByteBuffer.wrap(recvData, 0, len);
                long size = buf.getLong();

                if (size == sendData.length) {
                    // データ受信が完了したので、IFalpService#terminateFalp()を実行する
                    sender.terminateFalp();
                }
            }

            @Override
            public void sendDataPrepared(int emptySize) throws RemoteException {
                sendData(emptySize);
            }

            @Override
            public void shutdownFalpCompleted(int shutdownType, int chipState) throws RemoteException {
                resultCode = shutdownType;

                if (dialog.isShowing()) {
                    dialog.dismiss();
                }
                wakeUp();
            }

        };
        private int resultCode;

        // データ送信処理
        private void sendData(int size) {
            byte[] tmp;
            int dataSize = sendData.length - sendPos;

            if (dataSize == 0) {

                return;
            }

            if (dataSize > size) {
                dataSize = size;
            }
            tmp = new byte[dataSize];

            System.arraycopy(sendData, sendPos, tmp, 0, dataSize);
            int ret;
            try {
                ret = sender.sendData(tmp);
            } catch (RemoteException e) {
                return;
            }

            if (ret > 0) {
                sendPos += ret;
            }
        }

        @Override
        protected Void doInBackground(String... params) {
            sendData = params[0].getBytes();// 送信するデータ
            sender.connect();

            while (!sender.isConnected()) {
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                }
            }

            try {
                resultCode = sender.startInitiatorFalp(falpListener, AppIdentifier.APP_ID);
            } catch (RemoteException e) {
                e.printStackTrace();
            }
            waitEvent();

            return null;
        }

        @Override
        protected void onProgressUpdate(Integer... values) {

            String msg = "";
            if (values[0] == SUCESS_PROPOSEDINITIATORFALP) {
                if (mDialog.isShowing()) {
                    mDialog.dismiss();
                }
                msg = "相手アプリケーション起動後タッチ";

            } else {
                if (dialog.isShowing()) {
                    dialog.dismiss();
                }

                msg = "データ送信中";
            }

            dialog = new
                    AlertDialog.Builder(FelicaFALPInitiatorActivity.this).setMessage(msg)
                            .setNegativeButton("キャンセル", new
                                    DialogInterface.OnClickListener() {

                                        @Override
                                        public void onClick(DialogInterface dialog, int which) {
                                            // キャンセルボタンが押されたら、キャンセル処理を行う
                                            cancel(true);

                                        }
                                    }).create();

            dialog.show();

        }

        @Override
        protected void onPostExecute(Void result) {

            // 送信中ダイアログの非表示
            if (dialog.isShowing()) {
                dialog.dismiss();
            }

            if (resultCode != FalpServiceConst.FALP_RTN_OK) {
                // エラー
                showError(getApplicationContext(), resultCode);
            } else {
                showFinish(getApplicationContext());
            }

        }

        @Override
        protected void onCancelled() {
            // 通信キャンセル処理を行う
            sender.cancelFalp();
            if (dialog.isShowing()) {
                dialog.dismiss();
            }
        }

        // FALPイベント待ち合わせ用処理
        private synchronized void waitEvent() {

            condition = false;
            while (!condition) {
                try {
                    wait();
                } catch (Exception e) {

                }
            }
        }

        private synchronized void wakeUp() {

            condition = true;
            notifyAll();
        }

        public void showError(Context context, int returnCode) {

            String str = "";

            switch (returnCode) {
                case FalpServiceConst.FALP_RTN_SEND_DATASIZE_OVER:
                    str = "指定したデータのサイズが大きすぎるため、送信できませんでした。";
                    break;
                case FalpServiceConst.FALP_RTN_SECURITY_ERROR:
                    str = "他のアプリケーションがFeliCaチップを利用中です。" +
                            "FeliCaチップを利用する他のアプリケーションを終了してください。";
                    break;
                case FalpServiceConst.FALP_RTN_UNKNOWN_ERROR:
                    str = "FALPのプロトコルエラー、または、予期せぬエラーが発生しました。";
                    break;
                case FalpServiceConst.FALP_RTN_APPCODE_UNMATCH_ERROR:
                    str = "Targetに起動対象のアプリケーションが存在しなかった、または、" +
                            "Targetの起動対象アプリケーションからFALP通信の開始が許可されませんでした。";
                    break;
                case FalpServiceConst.FALP_RTN_TIMEOUT_ERROR:
                    str = "タイムアウトエラーが発生しました。" +
                            "端末が正しくかざされていることを確認し、再試行してください。";
                    break;
                case FalpServiceConst.FALP_RTN_BUFFER_FULL_ERROR:
                    str = "送信バッファに空きがない状態で、データの送信が要求されました。";
                    break;
                case FalpServiceConst.FALP_RTN_STATE_ERROR:
                    str = "呼び出せない状態で、メソッドが呼び出されました。";
                    break;
                case FalpServiceConst.FALP_RTN_INVALID_PARAM_ERROR:
                    str = "不正なパラメータが指定されてメソッドが呼び出されました。";
                    break;
                case FalpServiceConst.FALP_RTN_NEED_TO_CHIP_CLOSE_ERROR:
                    str = "FALP通信の終了処理に失敗しました。" +
                            "FeliCaチップのクローズ、またはリセットを実行してください。";
                    break;
                case FalpServiceConst.FALP_RTN_NOT_IC_CHIP_FORMATTING_ERROR:
                    str = "おサイフケータイの初期設定が未実施です。" +
                            "「おサイフケータイ アプリ」を起動してください。";
                    break;
                case FalpServiceConst.FALP_RTN_FELICA_NOT_AVAILABLE_ERROR:
                    str = "FeliCaチップがロックされています。" +
                            "「おサイフケータイ アプリ」を起動し、ロックを解除してください。";
                    break;
                case FalpServiceConst.FALP_RTN_RW_STOP_ERROR:
                    str = "リーダ／ライタ機能が使用できない状態です。";
                    break;
                case FalpServiceConst.FALP_RTN_TARGET_UNAVAILABLE_ERROR:
                    str = "対向のFeliCa対応端末がFALP通信を開始できない状態です。" +
                            "対向のFeliCa対応端末のFeliCaチップをクローズしてください。";
                    break;
                case FalpServiceConst.FALP_RTN_ILLEGAL_ACCESS_ERROR:
                    str = "呼び出し権限のないアプリからメソッドが呼び出されました。";
                    break;
                default:
                    str = "予期しないエラーが発生しました。";
                    break;
            }
            Toast.makeText(context, str, Toast.LENGTH_SHORT).show();

        }

        public void showFinish(Context context) {
            Toast.makeText(context, "FALP通信完了", Toast.LENGTH_SHORT).show();
        }
    }

}
