
package com.sample;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.IBinder;
import android.os.RemoteException;

import com.felicanetworks.falp.FalpListener;
import com.felicanetworks.falp.FalpServiceConst;
import com.felicanetworks.falp.IFalpService;

public class FelicaFALPInitiatorSender {

    private IFalpService falp;
    private Context context;
    private boolean connected;

    public FelicaFALPInitiatorSender(Context context) {
        this.context = context;
    }

    public void connect() {

        if (context == null) {
            // エラーハンドリング
            return;
        }

        if (connected) {
            // 接続済み
            // 接続処理
            return;
        }
        Intent intent = new Intent();
        intent.setAction(FalpServiceConst.ACTION_INITIATOR_START);
        intent.addCategory(FalpServiceConst.CATEGORY_INITIATOR_START);

        // IFalpSendIntentServiceへの接続要求
        // Context#startService()は実行しないでください
        if (context.bindService(intent, instance, Context.BIND_AUTO_CREATE) == false) {
            // エラーハンドリング
        }

    }

    public void disconnect() {

        if (context == null) {
            // エラーハンドリング
            return;
        }

        if (!connected) {
            // 非接続
            // 終了処理
            return;
        }

        context.unbindService(instance);

        // 接続状態を解除
        connected = false;
    }

    private final ServiceConnection instance = new ServiceConnection() {

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {

            // Falpとの接続が確立されたので、Falpインスタンスを取得する
            falp = IFalpService.Stub.asInterface(service);

            connected = true;
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
            connected = false;
            if (context != null) {
                context.unbindService(this);
            }
        }

    };

    public boolean isConnected() {
        return connected;
    }

    public int startInitiatorFalp(FalpListener listener, byte[] id) throws RemoteException {
        return falp.startInitiatorFalp(listener, id);
    }

    public void canceled() {

        disconnect();

    }

    public int getMaxSendSize() throws RemoteException {
        return falp.getMaxSendSize();
    }

    public int sendData(byte[] data) throws RemoteException {
        return falp.sendData(data);
    }

    public void terminateFalp() throws RemoteException {
        falp.terminateFalp();
    }

    public void cancelFalp() {
        try {
            falp.cancelFalp();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }
}
