
package com.sample;

import java.nio.ByteBuffer;

import android.app.Activity;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.RemoteException;
import android.util.Log;
import android.widget.TextView;

import com.felicanetworks.falp.FalpListener;
import com.felicanetworks.falp.FalpServiceConst;

public class FelicaFALPTargetActivity extends Activity {

    private FelicaFALPTargetSender sender;
    private TextView textv1;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.falp_target);

        textv1 = (TextView) findViewById(R.id.textv1);

        startTarget(getIntent());

    }

    private void startTarget(Intent intent) {
        // Intentをチェックし、FALP起動要求であるかをチェックする

        // <data>をチェックする
        String data = intent.getDataString();
        if (!AppIdentifier.FALP_INTENT_FILTER_DATA_FIELD.equals(data)) {
            // dataエラー
            return;
        }

        // ワンタイムパスワードを取得する
        Bundle bundle = intent.getExtras();
        if (!bundle.containsKey(FalpServiceConst.ONE_TIME_PSWD)) {
            // ワンタイムパスワードエラー
            return;
        }

        int password = bundle.getInt(FalpServiceConst.ONE_TIME_PSWD);

        sender = new FelicaFALPTargetSender(this);

        TargetTask task = new TargetTask();
        task.execute(password);
    }

    class TargetTask extends AsyncTask<Integer, Void, Void> {
        // FALP通信完了チェック用フラグ
        private boolean condition;
        // 受信完了データサイズ
        private byte[] recvData = new byte[0];

        private int lastNotifiedSize = 0;
        private ByteBuffer buf = ByteBuffer.allocate(16);

        private int resultCode;

        // FALP通信状態通知用リスナ
        private FalpListener listener = new FalpListener.Stub() {

            @Override
            public void shutdownFalpCompleted(int shutdownType, int chipState) throws RemoteException {
                resultCode = shutdownType;

                // 終了したので、run()での待ち合わせ処理を解除する
                wakeUp();
            }

            @Override
            public void sendDataPrepared(int emptySize) throws RemoteException {
                // 現在受信したサイズをInitiator側に通知
                // (サイズチェックはしない)
                int recvSize = recvData.length;
                if (recvSize <= lastNotifiedSize) {
                    // 受信データが更新されていないので即終了
                    return;
                }

                buf.clear();
                buf.putLong(recvSize);
                int size = buf.limit() - buf.position();

                if (emptySize <= size) {
                    // まとめてデータ送信ができないため終了
                    return;
                }

                byte[] tmp = new byte[size];
                buf.rewind();
                buf.get(tmp);

                int result = sender.sendData(tmp);
                Log.d("sample", "sendData:" + result);
                if (result > 0) {
                    // サイズ通知情報を更新
                    lastNotifiedSize = recvSize;
                }
            }

            @Override
            public void recvDataNotify(byte[] arg0, int arg1) throws RemoteException {

                // 受信データサイズをインクリメント
                int recvSize = recvData.length + arg1;

                // 受信データを更新
                byte[] newData = new byte[recvSize];
                System.arraycopy(recvData, 0, newData, 0, recvData.length);
                System.arraycopy(arg0, 0, newData, recvData.length, arg1);
                recvData = newData;
            }

            @Override
            public void receivedFalpHandshake() throws RemoteException {
                // 当該サンプルでは呼び出されない
            }

            @Override
            public void proposedInitiatorFalp() throws RemoteException {
                // Target側では呼び出されない
            }

            @Override
            public void completedInitiatorFalp() throws RemoteException {
                // Target側では呼び出されない
            }
        };

        @Override
        protected Void doInBackground(Integer... params) {

            sender.connect();
            while (!sender.isConnected()) {
                try {
                    Thread.sleep(10);
                } catch (InterruptedException e) {
                    // 無視する
                }
            }

            try {
                sender.acceptTargetFalp(listener, params[0]);

                waitEvent();
            } catch (RemoteException e) {
                // MFCエラー
            } finally {
                sender.disconnect();
            }
            return null;
        }

        // FALPイベント待ち合わせ用処理
        private synchronized void waitEvent() {

            condition = false;
            while (!condition) {
                try {
                    wait();
                } catch (Exception e) {

                }
            }
        }

        private synchronized void wakeUp() {

            condition = true;
            notifyAll();
        }

        @Override
        protected void onPostExecute(Void result) {
            // テキストに受信した文字列を表示

            if (resultCode == FalpServiceConst.FALP_RTN_OK) {
                textv1.setText(new String(recvData));
            }
        }
    }
}
