package com.sample;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.IBinder;
import android.os.RemoteException;

import com.felicanetworks.falp.android.intent.FalpSendIntentConst;
import com.felicanetworks.falp.android.intent.FalpSendIntentListener;
import com.felicanetworks.falp.android.intent.IFalpSendIntentService;

public class FelicaIntentSender{
    private IFalpSendIntentService falp;
    private Context context;
    private boolean connected;
    
    
    public FelicaIntentSender(Context context){
        this.context = context;
    }
    
    public void connect() {

        if (context == null) {
            // エラーハンドリング
            return;
        }

        if (connected) {
            // 接続済み
            // 接続処理
            return;
        }
        Intent intent = new Intent();
        intent.setAction(FalpSendIntentConst.ACTION_INITIATOR_START);
        intent.addCategory(FalpSendIntentConst.CATEGORY_INITIATOR_START);

        // IFalpSendIntentServiceへの接続要求
        // Context#startService()は実行しないでください
        if (context.bindService(intent, instance, Context.BIND_AUTO_CREATE) == false) {
            // エラーハンドリング
        }

    }

    public void disconnect() {

        if (context == null) {
            // エラーハンドリング
            return;
        }

        if (!connected) {
            // 非接続
            // 終了処理
            return;
        }

        context.unbindService(instance);

        // 接続状態を解除
        connected = false;
    }
    private final ServiceConnection instance = new ServiceConnection() {

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            
            // Falpとの接続が確立されたので、Falpインスタンスを取得する
            falp = IFalpSendIntentService.Stub.asInterface(service);

            connected = true;
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
            connected = false;
            if (context != null) {
                context.unbindService(this);
            }
        }

    };
    
    public boolean isConnected(){
        return connected;
    }
    
    public void sendIntent(FalpSendIntentListener listener,Intent intent){
        try {
            int result = falp.sendIntent(listener,intent);
            
            if(result == FalpSendIntentConst.FALP_RTN_STATE_ERROR){
                // 
            }
            
            if (result != FalpSendIntentConst.FALP_RTN_SEND_INTENT_ACCEPTED) {
             // Androidインテント実行要求に失敗した場合、エラー内容を表示する
                 return;
            }
            
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }
    
    public void canceled(){
        try {
            falp.cancel();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        
    }

    

}
